@@ -1,8 +1,13 @@
-/*
-* A simple wrapper for accessing data in R object.
-* Due to license issue, we cannot include R's header file, so use this simple wrapper instead.
-* However, if R changes the way it defines objects, this file will need to be updated as well.
-*/
+/*!
+ * Copyright (c) 2017 Microsoft Corporation. All rights reserved.
+ * Licensed under the MIT License. See LICENSE file in the project root for license information.
+ *
+ * \brief A simple wrapper for accessing data in R object.
+ *
+ * \note
+ * Due to license issue, we cannot include R's header file, so use this simple wrapper instead.
+ * However, if R changes the way it defines objects, this file will need to be updated as well.
+ */
 #ifndef R_OBJECT_HELPER_H_
 #define R_OBJECT_HELPER_H_
 
@@ -118,22 +123,26 @@ typedef struct VECTOR_SER {
 
 typedef union { VECTOR_SER s; double align; } SEXPREC_ALIGN;
 
-#define DATAPTR(x)  (((SEXPREC_ALIGN *) (x)) + 1)
+#define DATAPTR(x)  ((reinterpret_cast<SEXPREC_ALIGN*>(x)) + 1)
 
-#define R_CHAR_PTR(x)     ((char *) DATAPTR(x))
+#define R_CHAR_PTR(x)     (reinterpret_cast<char*>DATAPTR(x))
 
-#define R_INT_PTR(x)  ((int *) DATAPTR(x))
+#define R_INT_PTR(x)  (reinterpret_cast<int*> DATAPTR(x))
 
-#define R_REAL_PTR(x)     ((double *) DATAPTR(x))
+#define R_INT64_PTR(x)  (reinterpret_cast<int64_t*> DATAPTR(x))
 
-#define R_AS_INT(x) (*((int *) DATAPTR(x)))
+#define R_REAL_PTR(x)     (reinterpret_cast<double*> DATAPTR(x))
 
-#define R_IS_NULL(x) ((*(LGBM_SE)(x)).sxpinfo.type == 0)
+#define R_AS_INT(x) (*(reinterpret_cast<int*> DATAPTR(x)))
+
+#define R_AS_INT64(x) (*(reinterpret_cast<int64_t*> DATAPTR(x)))
+
+#define R_IS_NULL(x) ((*reinterpret_cast<LGBM_SE>(x)).sxpinfo.type == 0)
 
 // 64bit pointer
 #if INTPTR_MAX == INT64_MAX
 
-#define R_ADDR(x)  ((int64_t *) DATAPTR(x))
+#define R_ADDR(x)  (reinterpret_cast<int64_t*> DATAPTR(x))
 
 inline void R_SET_PTR(LGBM_SE x, void* ptr) {
   if (ptr == nullptr) {
@@ -147,7 +156,7 @@ inline void* R_GET_PTR(LGBM_SE x) {
   if (R_IS_NULL(x)) {
     return nullptr;
   } else {
-    auto ret = (void *)(R_ADDR(x)[0]);
+    auto ret = reinterpret_cast<void*>(R_ADDR(x)[0]);
     if (ret == NULL) {
       ret = nullptr;
     }
@@ -157,7 +166,7 @@ inline void* R_GET_PTR(LGBM_SE x) {
 
 #else
 
-#define R_ADDR(x)  ((int32_t *) DATAPTR(x))
+#define R_ADDR(x)  (reinterpret_cast<int32_t*> DATAPTR(x))
 
 inline void R_SET_PTR(LGBM_SE x, void* ptr) {
   if (ptr == nullptr) {
@@ -171,7 +180,7 @@ inline void* R_GET_PTR(LGBM_SE x) {
   if (R_IS_NULL(x)) {
     return nullptr;
   } else {
-    auto ret = (void *)(R_ADDR(x)[0]);
+    auto ret = reinterpret_cast<void*>(R_ADDR(x)[0]);
     if (ret == NULL) {
       ret = nullptr;
     }
@@ -181,4 +190,4 @@ inline void* R_GET_PTR(LGBM_SE x) {
 
 #endif
 
-#endif // R_OBJECT_HELPER_H_
+#endif  // R_OBJECT_HELPER_H_
