@@ -1,21 +1,25 @@
+/*!
+ * Copyright (c) 2016 Microsoft Corporation. All rights reserved.
+ * Licensed under the MIT License. See LICENSE file in the project root for license information.
+ */
 #ifndef LIGHTGBM_PREDICTOR_HPP_
 #define LIGHTGBM_PREDICTOR_HPP_
 
-#include <LightGBM/meta.h>
 #include <LightGBM/boosting.h>
-#include <LightGBM/utils/text_reader.h>
 #include <LightGBM/dataset.h>
-
+#include <LightGBM/meta.h>
 #include <LightGBM/utils/openmp_wrapper.h>
+#include <LightGBM/utils/text_reader.h>
 
-#include <map>
-#include <cstring>
-#include <cstdio>
-#include <vector>
-#include <utility>
-#include <functional>
 #include <string>
+#include <cstdio>
+#include <cstring>
+#include <functional>
+#include <map>
 #include <memory>
+#include <unordered_map>
+#include <utility>
+#include <vector>
 
 namespace LightGBM {
 
@@ -23,7 +27,7 @@ namespace LightGBM {
 * \brief Used to predict data with input model
 */
 class Predictor {
-public:
+ public:
   /*!
   * \brief Constructor
   * \param boosting Input boosting model
@@ -35,7 +39,6 @@ public:
   Predictor(Boosting* boosting, int num_iteration,
             bool is_raw_score, bool predict_leaf_index, bool predict_contrib,
             bool early_stop, int early_stop_freq, double early_stop_margin) {
-
     early_stop_ = CreatePredictionEarlyStopInstance("none", LightGBM::PredictionEarlyStopConfig());
     if (early_stop && !boosting->NeedAccuratePrediction()) {
       PredictionEarlyStopConfig pred_early_stop_config;
@@ -76,16 +79,16 @@ public:
         }
       };
     } else if (predict_contrib) {
-	    predict_fun_ = [=](const std::vector<std::pair<int, double>>& features, double* output) {
-	      int tid = omp_get_thread_num();
-			CopyToPredictBuffer(predict_buf_[tid].data(), features);
+        predict_fun_ = [=](const std::vector<std::pair<int, double>>& features, double* output) {
+          int tid = omp_get_thread_num();
+          CopyToPredictBuffer(predict_buf_[tid].data(), features);
           // get result for leaf index
           boosting_->PredictContrib(predict_buf_[tid].data(), output, &early_stop_);
           ClearPredictBuffer(predict_buf_[tid].data(), predict_buf_[tid].size(), features);
         };
     } else {
       if (is_raw_score) {
-		predict_fun_ = [=](const std::vector<std::pair<int, double>>& features, double* output) {
+        predict_fun_ = [=](const std::vector<std::pair<int, double>>& features, double* output) {
           int tid = omp_get_thread_num();
           if (num_feature_ > kFeatureThreshold && features.size() < KSparseThreshold) {
             auto buf = CopyToPredictMap(features);
@@ -97,7 +100,7 @@ public:
           }
         };
       } else {
-		predict_fun_ = [=](const std::vector<std::pair<int, double>>& features, double* output) {
+        predict_fun_ = [=](const std::vector<std::pair<int, double>>& features, double* output) {
           int tid = omp_get_thread_num();
           if (num_feature_ > kFeatureThreshold && features.size() < KSparseThreshold) {
             auto buf = CopyToPredictMap(features);
@@ -173,7 +176,7 @@ public:
             (*feature)[i].first = feature_names_map_[(*feature)[i].first];
             ++i;
           } else {
-            //move the non-used features to the end of the feature vector
+            // move the non-used features to the end of the feature vector
             std::swap((*feature)[i], (*feature)[--j]);
           }
         }
@@ -181,8 +184,8 @@ public:
       }
     };
 
-	std::function<void(data_size_t, const std::vector<std::string>&)> process_fun = [&]
-	(data_size_t, const std::vector<std::string>& lines) {
+    std::function<void(data_size_t, const std::vector<std::string>&)> process_fun = [&]
+    (data_size_t, const std::vector<std::string>& lines) {
       std::vector<std::pair<int, double>> oneline_features;
       std::vector<std::string> result_to_write(lines.size());
       OMP_INIT_EX();
@@ -208,8 +211,7 @@ public:
     predict_data_reader.ReadAllAndProcessParallel(process_fun);
   }
 
-private:
-
+ private:
   void CopyToPredictBuffer(double* pred_buf, const std::vector<std::pair<int, double>>& features) {
     int loop_size = static_cast<int>(features.size());
     for (int i = 0; i < loop_size; ++i) {
