@@ -1,20 +1,19 @@
+/*!
+ * Copyright (c) 2016 Microsoft Corporation. All rights reserved.
+ * Licensed under the MIT License. See LICENSE file in the project root for license information.
+ */
 #include "gbdt.h"
 
+#include <LightGBM/metric.h>
+#include <LightGBM/network.h>
+#include <LightGBM/objective_function.h>
+#include <LightGBM/prediction_early_stop.h>
+#include <LightGBM/utils/common.h>
 #include <LightGBM/utils/openmp_wrapper.h>
 
-#include <LightGBM/utils/common.h>
-#include <LightGBM/objective_function.h>
-#include <LightGBM/metric.h>
-#include <LightGBM/prediction_early_stop.h>
-#include <LightGBM/network.h>
-
-#include <ctime>
-
-#include <sstream>
 #include <chrono>
-#include <string>
-#include <vector>
-#include <utility>
+#include <ctime>
+#include <sstream>
 
 namespace LightGBM {
 
@@ -23,14 +22,15 @@ GBDT::GBDT() : iter_(0),
 train_data_(nullptr),
 objective_function_(nullptr),
 early_stopping_round_(0),
+es_first_metric_only_(false),
 max_feature_idx_(0),
 num_tree_per_iteration_(1),
 num_class_(1),
 num_iteration_for_pred_(0),
 shrinkage_rate_(0.1f),
 num_init_iteration_(0),
-need_re_bagging_(false) {
-
+need_re_bagging_(false),
+balanced_bagging_(false) {
   #pragma omp parallel
   #pragma omp master
   {
@@ -41,7 +41,6 @@ need_re_bagging_(false) {
 }
 
 GBDT::~GBDT() {
-
 }
 
 void GBDT::Init(const Config* config, const Dataset* train_data, const ObjectiveFunction* objective_function,
@@ -54,10 +53,11 @@ void GBDT::Init(const Config* config, const Dataset* train_data, const Objective
   num_class_ = config->num_class;
   config_ = std::unique_ptr<Config>(new Config(*config));
   early_stopping_round_ = config_->early_stopping_round;
+  es_first_metric_only_ = config_->first_metric_only;
   shrinkage_rate_ = config_->learning_rate;
 
   std::string forced_splits_path = config->forcedsplits_filename;
-  //load forced_splits file
+  // load forced_splits file
   if (forced_splits_path != "") {
       std::ifstream forced_splits_file(forced_splits_path.c_str());
       std::stringstream buffer;
@@ -103,6 +103,7 @@ void GBDT::Init(const Config* config, const Dataset* train_data, const Objective
   // get feature names
   feature_names_ = train_data_->feature_names();
   feature_infos_ = train_data_->feature_infos();
+  monotone_constraints_ = config->monotone_constraints;
 
   // if need bagging, create buffer
   ResetBaggingConfig(config_.get(), true);
@@ -132,20 +133,18 @@ void GBDT::AddValidDataset(const Dataset* valid_data,
   }
   valid_score_updater_.push_back(std::move(new_score_updater));
   valid_metrics_.emplace_back();
-  if (early_stopping_round_ > 0) {
-    best_iter_.emplace_back();
-    best_score_.emplace_back();
-    best_msg_.emplace_back();
-  }
   for (const auto& metric : valid_metrics) {
     valid_metrics_.back().push_back(metric);
-    if (early_stopping_round_ > 0) {
-      best_iter_.back().push_back(0);
-      best_score_.back().push_back(kMinScore);
-      best_msg_.back().emplace_back();
-    }
   }
   valid_metrics_.back().shrink_to_fit();
+
+  if (early_stopping_round_ > 0) {
+    auto num_metrics = valid_metrics.size();
+    if (es_first_metric_only_) { num_metrics = 1; }
+    best_iter_.emplace_back(num_metrics, 0);
+    best_score_.emplace_back(num_metrics, kMinScore);
+    best_msg_.emplace_back(num_metrics);
+  }
 }
 
 void GBDT::Boosting() {
@@ -158,7 +157,7 @@ void GBDT::Boosting() {
     GetGradients(GetTrainingScore(&num_score), gradients_.data(), hessians_.data());
 }
 
-data_size_t GBDT::BaggingHelper(Random& cur_rand, data_size_t start, data_size_t cnt, data_size_t* buffer) {
+data_size_t GBDT::BaggingHelper(Random* cur_rand, data_size_t start, data_size_t cnt, data_size_t* buffer) {
   if (cnt <= 0) {
     return 0;
   }
@@ -169,7 +168,7 @@ data_size_t GBDT::BaggingHelper(Random& cur_rand, data_size_t start, data_size_t
   // random bagging, minimal unit is one record
   for (data_size_t i = 0; i < cnt; ++i) {
     float prob = (bag_data_cnt - cur_left_cnt) / static_cast<float>(cnt - i);
-    if (cur_rand.NextFloat() < prob) {
+    if (cur_rand->NextFloat() < prob) {
       buffer[cur_left_cnt++] = start + i;
     } else {
       right_buffer[cur_right_cnt++] = start + i;
@@ -179,6 +178,35 @@ data_size_t GBDT::BaggingHelper(Random& cur_rand, data_size_t start, data_size_t
   return cur_left_cnt;
 }
 
+data_size_t GBDT::BalancedBaggingHelper(Random* cur_rand, data_size_t start, data_size_t cnt, data_size_t* buffer) {
+  if (cnt <= 0) {
+    return 0;
+  }
+  auto label_ptr = train_data_->metadata().label();
+  data_size_t cur_left_cnt = 0;
+  data_size_t cur_right_pos = cnt - 1;
+  // from right to left
+  auto right_buffer = buffer;
+  // random bagging, minimal unit is one record
+  for (data_size_t i = 0; i < cnt; ++i) {
+    bool is_pos = label_ptr[start + i] > 0;
+    bool is_in_bag = false;
+    if (is_pos) {
+      is_in_bag = cur_rand->NextFloat() < config_->pos_bagging_fraction;
+    } else {
+      is_in_bag = cur_rand->NextFloat() < config_->neg_bagging_fraction;
+    }
+    if (is_in_bag) {
+      buffer[cur_left_cnt++] = start + i;
+    } else {
+      right_buffer[cur_right_pos--] = start + i;
+    }
+  }
+  // reverse right buffer
+  std::reverse(buffer + cur_left_cnt, buffer + cnt);
+  return cur_left_cnt;
+}
+
 void GBDT::Bagging(int iter) {
   // if need bagging
   if ((bag_data_cnt_ < num_data_ && iter % config_->bagging_freq == 0)
@@ -188,7 +216,7 @@ void GBDT::Bagging(int iter) {
     data_size_t inner_size = (num_data_ + num_threads_ - 1) / num_threads_;
     if (inner_size < min_inner_size) { inner_size = min_inner_size; }
     OMP_INIT_EX();
-    #pragma omp parallel for schedule(static,1)
+    #pragma omp parallel for schedule(static, 1)
     for (int i = 0; i < num_threads_; ++i) {
       OMP_LOOP_EX_BEGIN();
       left_cnts_buf_[i] = 0;
@@ -198,7 +226,12 @@ void GBDT::Bagging(int iter) {
       data_size_t cur_cnt = inner_size;
       if (cur_start + cur_cnt > num_data_) { cur_cnt = num_data_ - cur_start; }
       Random cur_rand(config_->bagging_seed + iter * num_threads_ + i);
-      data_size_t cur_left_count = BaggingHelper(cur_rand, cur_start, cur_cnt, tmp_indices_.data() + cur_start);
+      data_size_t cur_left_count = 0;
+      if (balanced_bagging_) {
+        cur_left_count = BalancedBaggingHelper(&cur_rand, cur_start, cur_cnt, tmp_indices_.data() + cur_start);
+      } else {
+        cur_left_count = BaggingHelper(&cur_rand, cur_start, cur_cnt, tmp_indices_.data() + cur_start);
+      }
       offsets_buf_[i] = cur_start;
       left_cnts_buf_[i] = cur_left_count;
       right_cnts_buf_[i] = cur_cnt - cur_left_count;
@@ -308,15 +341,17 @@ double ObtainAutomaticInitialScore(const ObjectiveFunction* fobj, int class_id)
   return init_score;
 }
 
-double GBDT::BoostFromAverage(int class_id) {
+double GBDT::BoostFromAverage(int class_id, bool update_scorer) {
   // boosting from average label; or customized "average" if implemented for the current objective
   if (models_.empty() && !train_score_updater_->has_init_score() && objective_function_ != nullptr) {
     if (config_->boost_from_average || (train_data_ != nullptr && train_data_->num_features() == 0)) {
       double init_score = ObtainAutomaticInitialScore(objective_function_, class_id);
       if (std::fabs(init_score) > kEpsilon) {
-        train_score_updater_->AddScore(init_score, class_id);
-        for (auto& score_updater : valid_score_updater_) {
-          score_updater->AddScore(init_score, class_id);
+        if (update_scorer) {
+          train_score_updater_->AddScore(init_score, class_id);
+          for (auto& score_updater : valid_score_updater_) {
+            score_updater->AddScore(init_score, class_id);
+          }
         }
         Log::Info("Start training from score %lf", init_score);
         return init_score;
@@ -335,7 +370,7 @@ bool GBDT::TrainOneIter(const score_t* gradients, const score_t* hessians) {
   // boosting first
   if (gradients == nullptr || hessians == nullptr) {
     for (int cur_tree_id = 0; cur_tree_id < num_tree_per_iteration_; ++cur_tree_id) {
-      init_scores[cur_tree_id] = BoostFromAverage(cur_tree_id);
+      init_scores[cur_tree_id] = BoostFromAverage(cur_tree_id, true);
     }
     Boosting();
     gradients = gradients_.data();
@@ -365,7 +400,9 @@ bool GBDT::TrainOneIter(const score_t* gradients, const score_t* hessians) {
 
     if (new_tree->num_leaves() > 1) {
       should_continue = true;
-      tree_learner_->RenewTreeOutput(new_tree.get(), objective_function_, train_score_updater_->score() + bias,
+      auto score_ptr = train_score_updater_->score() + bias;
+      auto residual_getter = [score_ptr](const label_t* label, int i) {return static_cast<double>(label[i]) - score_ptr[i]; };
+      tree_learner_->RenewTreeOutput(new_tree.get(), objective_function_, residual_getter,
                                      num_data_, bag_data_indices_.data(), bag_data_cnt_);
       // shrinkage by learning rate
       new_tree->Shrinkage(shrinkage_rate_);
@@ -449,7 +486,6 @@ bool GBDT::EvalAndCheckEarlyStopping() {
 }
 
 void GBDT::UpdateScore(const Tree* tree, const int cur_tree_id) {
-
   // update training score
   if (!is_use_subset_) {
     train_score_updater_->AddScore(tree_learner_.get(), tree, cur_tree_id);
@@ -514,6 +550,7 @@ std::string GBDT::OutputMetric(int iter) {
             msg_buf << tmp_buf.str() << '\n';
           }
         }
+        if (es_first_metric_only_ && j > 0) { continue; }
         if (ret.empty() && early_stopping_round_ > 0) {
           auto cur_score = valid_metrics_[i][j]->factor_to_bigger_better() * test_scores.back();
           if (cur_score > best_score_[i][j]) {
@@ -597,7 +634,7 @@ void GBDT::GetPredictAt(int data_idx, double* out_result, int64_t* out_len) {
     num_data = valid_score_updater_[used_idx]->num_data();
     *out_len = static_cast<int64_t>(num_data) * num_class_;
   }
-  if (objective_function_ != nullptr && !average_output_) {
+  if (objective_function_ != nullptr) {
     #pragma omp parallel for schedule(static)
     for (data_size_t i = 0; i < num_data; ++i) {
       std::vector<double> tree_pred(num_tree_per_iteration_);
@@ -622,7 +659,6 @@ void GBDT::GetPredictAt(int data_idx, double* out_result, int64_t* out_len) {
 
 void GBDT::ResetTrainingData(const Dataset* train_data, const ObjectiveFunction* objective_function,
                              const std::vector<const Metric*>& training_metrics) {
-
   if (train_data != train_data_ && !train_data_->CheckAlign(*train_data)) {
     Log::Fatal("Cannot reset training data, since new training data has different bin mappers");
   }
@@ -690,9 +726,25 @@ void GBDT::ResetConfig(const Config* config) {
 
 void GBDT::ResetBaggingConfig(const Config* config, bool is_change_dataset) {
   // if need bagging, create buffer
-  if (config->bagging_fraction < 1.0 && config->bagging_freq > 0) {
-    bag_data_cnt_ =
-      static_cast<data_size_t>(config->bagging_fraction * num_data_);
+  data_size_t num_pos_data = 0;
+  if (objective_function_ != nullptr) {
+    num_pos_data = objective_function_->NumPositiveData();
+  }
+  bool balance_bagging_cond = (config->pos_bagging_fraction < 1.0 || config->neg_bagging_fraction < 1.0) && (num_pos_data > 0);
+  if ((config->bagging_fraction < 1.0 || balance_bagging_cond) && config->bagging_freq > 0) {
+    need_re_bagging_ = false;
+    if (!is_change_dataset &&
+      config_.get() != nullptr && config_->bagging_fraction == config->bagging_fraction && config_->bagging_freq == config->bagging_freq
+      && config_->pos_bagging_fraction == config->pos_bagging_fraction && config_->neg_bagging_fraction == config->neg_bagging_fraction) {
+      return;
+    }
+    if (balance_bagging_cond) {
+      balanced_bagging_ = true;
+      bag_data_cnt_ = static_cast<data_size_t>(num_pos_data * config->pos_bagging_fraction)
+                      + static_cast<data_size_t>((num_data_ - num_pos_data) * config->neg_bagging_fraction);
+    } else {
+      bag_data_cnt_ = static_cast<data_size_t>(config->bagging_fraction * num_data_);
+    }
     bag_data_indices_.resize(num_data_);
     tmp_indices_.resize(num_data_);
 
@@ -702,7 +754,7 @@ void GBDT::ResetBaggingConfig(const Config* config, bool is_change_dataset) {
     left_write_pos_buf_.resize(num_threads_);
     right_write_pos_buf_.resize(num_threads_);
 
-    double average_bag_rate = config->bagging_fraction / config->bagging_freq;
+    double average_bag_rate = (bag_data_cnt_ / num_data_) / config->bagging_freq;
     int sparse_group = 0;
     for (int i = 0; i < train_data_->num_feature_groups(); ++i) {
       if (train_data_->FeatureGroupIsSparse(i)) {
@@ -722,9 +774,7 @@ void GBDT::ResetBaggingConfig(const Config* config, bool is_change_dataset) {
       Log::Debug("Use subset for bagging");
     }
 
-    if (is_change_dataset) {
-      need_re_bagging_ = true;
-    }
+    need_re_bagging_ = true;
 
     if (is_use_subset_ && bag_data_cnt_ < num_data_) {
       if (objective_function_ == nullptr) {
