@@ -1,17 +1,20 @@
+/*!
+ * Copyright (c) 2016 Microsoft Corporation. All rights reserved.
+ * Licensed under the MIT License. See LICENSE file in the project root for license information.
+ */
 #include <LightGBM/dataset.h>
+
 #include <LightGBM/feature_group.h>
+#include <LightGBM/utils/array_args.h>
 #include <LightGBM/utils/openmp_wrapper.h>
 #include <LightGBM/utils/threading.h>
-#include <LightGBM/utils/array_args.h>
 
+#include <limits>
 #include <chrono>
 #include <cstdio>
-#include <unordered_map>
-#include <limits>
-#include <vector>
-#include <utility>
-#include <string>
 #include <sstream>
+#include <unordered_map>
+
 
 namespace LightGBM {
 
@@ -57,9 +60,9 @@ int GetConfilctCount(const std::vector<bool>& mark, const int* indices, int num_
   }
   return ret;
 }
-void MarkUsed(std::vector<bool>& mark, const int* indices, int num_indices) {
+void MarkUsed(std::vector<bool>* mark, const int* indices, int num_indices) {
   for (int i = 0; i < num_indices; ++i) {
-    mark[indices[i]] = true;
+    mark->at(indices[i]) = true;
   }
 }
 
@@ -86,7 +89,7 @@ std::vector<std::vector<int>> FindGroups(const std::vector<std::unique_ptr<BinMa
     bool need_new_group = true;
     std::vector<int> available_groups;
     for (int gid = 0; gid < static_cast<int>(features_in_group.size()); ++gid) {
-      if (group_non_zero_cnt[gid] + cur_non_zero_cnt <= total_sample_cnt + max_error_cnt){
+      if (group_non_zero_cnt[gid] + cur_non_zero_cnt <= total_sample_cnt + max_error_cnt) {
         if (!is_use_gpu || group_num_bin[gid] + bin_mappers[fidx]->num_bin() + (bin_mappers[fidx]->GetDefaultBin() == 0 ? -1 : 0)
             <= gpu_max_bin_per_group) {
           available_groups.push_back(gid);
@@ -113,7 +116,7 @@ std::vector<std::vector<int>> FindGroups(const std::vector<std::unique_ptr<BinMa
         features_in_group[gid].push_back(fidx);
         group_conflict_cnt[gid] += cnt;
         group_non_zero_cnt[gid] += cur_non_zero_cnt - cnt;
-        MarkUsed(conflict_marks[gid], sample_indices[fidx], num_per_col[fidx]);
+        MarkUsed(&conflict_marks[gid], sample_indices[fidx], num_per_col[fidx]);
         if (is_use_gpu) {
           group_num_bin[gid] += bin_mappers[fidx]->num_bin() + (bin_mappers[fidx]->GetDefaultBin() == 0 ? -1 : 0);
         }
@@ -125,7 +128,7 @@ std::vector<std::vector<int>> FindGroups(const std::vector<std::unique_ptr<BinMa
       features_in_group.back().push_back(fidx);
       group_conflict_cnt.push_back(0);
       conflict_marks.emplace_back(total_sample_cnt, false);
-      MarkUsed(conflict_marks.back(), sample_indices[fidx], num_per_col[fidx]);
+      MarkUsed(&(conflict_marks.back()), sample_indices[fidx], num_per_col[fidx]);
       group_non_zero_cnt.emplace_back(cur_non_zero_cnt);
       if (is_use_gpu) {
         group_num_bin.push_back(1 + bin_mappers[fidx]->num_bin() + (bin_mappers[fidx]->GetDefaultBin() == 0 ? -1 : 0));
@@ -135,7 +138,7 @@ std::vector<std::vector<int>> FindGroups(const std::vector<std::unique_ptr<BinMa
   return features_in_group;
 }
 
-std::vector<std::vector<int>> FastFeatureBundling(std::vector<std::unique_ptr<BinMapper>>& bin_mappers,
+std::vector<std::vector<int>> FastFeatureBundling(const std::vector<std::unique_ptr<BinMapper>>& bin_mappers,
                                                   int** sample_indices,
                                                   const int* num_per_col,
                                                   size_t total_sample_cnt,
@@ -149,16 +152,16 @@ std::vector<std::vector<int>> FastFeatureBundling(std::vector<std::unique_ptr<Bi
   // filter is based on sampling data, so decrease its range
   const data_size_t filter_cnt = static_cast<data_size_t>(static_cast<double>(0.95 * min_data) / num_data * total_sample_cnt);
   const data_size_t max_error_cnt = static_cast<data_size_t>(total_sample_cnt * max_conflict_rate);
-  int cur_used_feature_cnt = 0;
   std::vector<size_t> feature_non_zero_cnt;
+  feature_non_zero_cnt.reserve(used_features.size());
   // put dense feature first
   for (auto fidx : used_features) {
     feature_non_zero_cnt.emplace_back(num_per_col[fidx]);
-    ++cur_used_feature_cnt;
   }
   // sort by non zero cnt
   std::vector<int> sorted_idx;
-  for (int i = 0; i < cur_used_feature_cnt; ++i) {
+  sorted_idx.reserve(used_features.size());
+  for (int i = 0; i < static_cast<int>(used_features.size()); ++i) {
     sorted_idx.emplace_back(i);
   }
   // sort by non zero cnt, bigger first
@@ -168,6 +171,7 @@ std::vector<std::vector<int>> FastFeatureBundling(std::vector<std::unique_ptr<Bi
   });
 
   std::vector<int> feature_order_by_cnt;
+  feature_order_by_cnt.reserve(sorted_idx.size());
   for (auto sidx : sorted_idx) {
     feature_order_by_cnt.push_back(used_features[sidx]);
   }
@@ -187,7 +191,7 @@ std::vector<std::vector<int>> FastFeatureBundling(std::vector<std::unique_ptr<Bi
         cnt_non_zero += static_cast<int>(num_data * (1.0f - bin_mappers[fidx]->sparse_rate()));
       }
       double sparse_rate = 1.0f - static_cast<double>(cnt_non_zero) / (num_data);
-      // take apart small sparse group, due it will not gain on speed 
+      // take apart small sparse group, due it will not gain on speed
       if (sparse_rate >= sparse_threshold && is_enable_sparse) {
         for (size_t j = 0; j < features_in_group[i].size(); ++j) {
           const int fidx = features_in_group[i][j];
@@ -210,18 +214,18 @@ std::vector<std::vector<int>> FastFeatureBundling(std::vector<std::unique_ptr<Bi
 }
 
 void Dataset::Construct(
-  std::vector<std::unique_ptr<BinMapper>>& bin_mappers,
+  std::vector<std::unique_ptr<BinMapper>>* bin_mappers,
+  const std::vector<std::vector<double>>& forced_bins,
   int** sample_non_zero_indices,
   const int* num_per_col,
   size_t total_sample_cnt,
   const Config& io_config) {
-
-  num_total_features_ = static_cast<int>(bin_mappers.size());
+  num_total_features_ = static_cast<int>(bin_mappers->size());
   sparse_threshold_ = io_config.sparse_threshold;
   // get num_features
   std::vector<int> used_features;
-  for (int i = 0; i < static_cast<int>(bin_mappers.size()); ++i) {
-    if (bin_mappers[i] != nullptr && !bin_mappers[i]->is_trival()) {
+  for (int i = 0; i < static_cast<int>(bin_mappers->size()); ++i) {
+    if (bin_mappers->at(i) != nullptr && !bin_mappers->at(i)->is_trivial()) {
       used_features.emplace_back(i);
     }
   }
@@ -231,7 +235,7 @@ void Dataset::Construct(
   auto features_in_group = NoGroup(used_features);
 
   if (io_config.enable_bundle && !used_features.empty()) {
-    features_in_group = FastFeatureBundling(bin_mappers,
+    features_in_group = FastFeatureBundling(*bin_mappers,
                                             sample_non_zero_indices, num_per_col, total_sample_cnt,
                                             used_features, io_config.max_conflict_rate,
                                             num_data_, io_config.min_data_in_leaf,
@@ -259,11 +263,11 @@ void Dataset::Construct(
       real_feature_idx_[cur_fidx] = real_fidx;
       feature2group_[cur_fidx] = i;
       feature2subfeature_[cur_fidx] = j;
-      cur_bin_mappers.emplace_back(bin_mappers[real_fidx].release());
+      cur_bin_mappers.emplace_back(bin_mappers->at(real_fidx).release());
       ++cur_fidx;
     }
     feature_groups_.emplace_back(std::unique_ptr<FeatureGroup>(
-      new FeatureGroup(cur_cnt_features, cur_bin_mappers, num_data_, sparse_threshold_,
+      new FeatureGroup(cur_cnt_features, &cur_bin_mappers, num_data_, sparse_threshold_,
                        io_config.is_enable_sparse)));
   }
   feature_groups_.shrink_to_fit();
@@ -290,6 +294,75 @@ void Dataset::Construct(
     }
   }
 
+  if (!io_config.monotone_constraints.empty()) {
+    CHECK(static_cast<size_t>(num_total_features_) == io_config.monotone_constraints.size());
+    monotone_types_.resize(num_features_);
+    for (int i = 0; i < num_total_features_; ++i) {
+      int inner_fidx = InnerFeatureIndex(i);
+      if (inner_fidx >= 0) {
+        monotone_types_[inner_fidx] = io_config.monotone_constraints[i];
+      }
+    }
+    if (ArrayArgs<int8_t>::CheckAllZero(monotone_types_)) {
+      monotone_types_.clear();
+    }
+  }
+  if (!io_config.feature_contri.empty()) {
+    CHECK(static_cast<size_t>(num_total_features_) == io_config.feature_contri.size());
+    feature_penalty_.resize(num_features_);
+    for (int i = 0; i < num_total_features_; ++i) {
+      int inner_fidx = InnerFeatureIndex(i);
+      if (inner_fidx >= 0) {
+        feature_penalty_[inner_fidx] = std::max(0.0, io_config.feature_contri[i]);
+      }
+    }
+    if (ArrayArgs<double>::CheckAll(feature_penalty_, 1.0)) {
+      feature_penalty_.clear();
+    }
+  }
+  if (!io_config.max_bin_by_feature.empty()) {
+    CHECK(static_cast<size_t>(num_total_features_) == io_config.max_bin_by_feature.size());
+    CHECK(*(std::min_element(io_config.max_bin_by_feature.begin(), io_config.max_bin_by_feature.end())) > 1);
+    max_bin_by_feature_.resize(num_total_features_);
+    max_bin_by_feature_.assign(io_config.max_bin_by_feature.begin(), io_config.max_bin_by_feature.end());
+  }
+  forced_bin_bounds_ = forced_bins;
+  max_bin_ = io_config.max_bin;
+  min_data_in_bin_ = io_config.min_data_in_bin;
+  bin_construct_sample_cnt_ = io_config.bin_construct_sample_cnt;
+  use_missing_ = io_config.use_missing;
+  zero_as_missing_ = io_config.zero_as_missing;
+}
+
+void Dataset::ResetConfig(const char* parameters) {
+  auto param = Config::Str2Map(parameters);
+  Config io_config;
+  io_config.Set(param);
+  if (param.count("max_bin") && io_config.max_bin != max_bin_) {
+    Log::Warning("Cannot change max_bin after constructed Dataset handle.");
+  }
+  if (param.count("max_bin_by_feature") && io_config.max_bin_by_feature != max_bin_by_feature_) {
+    Log::Warning("Cannot change max_bin_by_feature after constructed Dataset handle.");
+  }
+  if (param.count("bin_construct_sample_cnt") && io_config.bin_construct_sample_cnt != bin_construct_sample_cnt_) {
+    Log::Warning("Cannot change bin_construct_sample_cnt after constructed Dataset handle.");
+  }
+  if (param.count("min_data_in_bin") && io_config.min_data_in_bin != min_data_in_bin_) {
+    Log::Warning("Cannot change min_data_in_bin after constructed Dataset handle.");
+  }
+  if (param.count("use_missing") && io_config.use_missing != use_missing_) {
+    Log::Warning("Cannot change use_missing after constructed Dataset handle.");
+  }
+  if (param.count("zero_as_missing") && io_config.zero_as_missing != zero_as_missing_) {
+    Log::Warning("Cannot change zero_as_missing after constructed Dataset handle.");
+  }
+  if (param.count("sparse_threshold") && io_config.sparse_threshold != sparse_threshold_) {
+    Log::Warning("Cannot change sparse_threshold after constructed Dataset handle.");
+  }
+  if (param.count("forcedbins_filename")) {
+    Log::Warning("Cannot change forced bins after constructed Dataset handle.");
+  }
+
   if (!io_config.monotone_constraints.empty()) {
     CHECK(static_cast<size_t>(num_total_features_) == io_config.monotone_constraints.size());
     monotone_types_.resize(num_features_);
@@ -346,7 +419,7 @@ void Dataset::CopyFeatureMapperFrom(const Dataset* dataset) {
     }
     feature_groups_.emplace_back(new FeatureGroup(
       dataset->feature_groups_[i]->num_feature_,
-      bin_mappers,
+      &bin_mappers,
       num_data_,
       dataset->feature_groups_[i]->is_sparse_));
   }
@@ -363,6 +436,7 @@ void Dataset::CopyFeatureMapperFrom(const Dataset* dataset) {
   group_feature_cnt_ = dataset->group_feature_cnt_;
   monotone_types_ = dataset->monotone_types_;
   feature_penalty_ = dataset->feature_penalty_;
+  forced_bin_bounds_ = dataset->forced_bin_bounds_;
 }
 
 void Dataset::CreateValid(const Dataset* dataset) {
@@ -379,9 +453,9 @@ void Dataset::CreateValid(const Dataset* dataset) {
     bin_mappers.emplace_back(new BinMapper(*(dataset->FeatureBinMapper(i))));
     feature_groups_.emplace_back(new FeatureGroup(
       1,
-      bin_mappers,
+      &bin_mappers,
       num_data_,
-      dataset->sparse_threshold_,
+      sparse_threshold_,
       is_enable_sparse));
     feature2group_.push_back(i);
     feature2subfeature_.push_back(0);
@@ -417,6 +491,7 @@ void Dataset::CreateValid(const Dataset* dataset) {
   }
   monotone_types_ = dataset->monotone_types_;
   feature_penalty_ = dataset->feature_penalty_;
+  forced_bin_bounds_ = dataset->forced_bin_bounds_;
 }
 
 void Dataset::ReSize(data_size_t num_data) {
@@ -521,6 +596,9 @@ bool Dataset::GetDoubleField(const char* field_name, data_size_t* out_len, const
   if (name == std::string("init_score")) {
     *out_ptr = metadata_.init_score();
     *out_len = static_cast<data_size_t>(metadata_.num_init_score());
+  } else if (name == std::string("feature_penalty")) {
+    *out_ptr = feature_penalty_.data();
+    *out_len = static_cast<data_size_t>(feature_penalty_.size());
   } else {
     return false;
   }
@@ -539,6 +617,18 @@ bool Dataset::GetIntField(const char* field_name, data_size_t* out_len, const in
   return true;
 }
 
+bool Dataset::GetInt8Field(const char* field_name, data_size_t* out_len, const int8_t** out_ptr) {
+  std::string name(field_name);
+  name = Common::Trim(name);
+  if (name == std::string("monotone_constraints")) {
+    *out_ptr = monotone_types_.data();
+    *out_len = static_cast<data_size_t>(monotone_types_.size());
+  } else {
+    return false;
+  }
+  return true;
+}
+
 void Dataset::SaveBinaryFile(const char* bin_filename) {
   if (bin_filename != nullptr
       && std::string(bin_filename) == data_filename_) {
@@ -568,19 +658,29 @@ void Dataset::SaveBinaryFile(const char* bin_filename) {
     writer->Write(binary_file_token, size_of_token);
     // get size of header
     size_t size_of_header = sizeof(num_data_) + sizeof(num_features_) + sizeof(num_total_features_)
-      + sizeof(int) * num_total_features_ + sizeof(label_idx_) + sizeof(num_groups_)
+      + sizeof(int) * num_total_features_ + sizeof(label_idx_) + sizeof(num_groups_) + sizeof(sparse_threshold_)
       + 3 * sizeof(int) * num_features_ + sizeof(uint64_t) * (num_groups_ + 1) + 2 * sizeof(int) * num_groups_ + sizeof(int8_t) * num_features_
-      + sizeof(double) * num_features_;
+      + sizeof(double) * num_features_ + sizeof(int32_t) * num_total_features_ + sizeof(int) * 3 + sizeof(bool) * 2;
     // size of feature names
     for (int i = 0; i < num_total_features_; ++i) {
       size_of_header += feature_names_[i].size() + sizeof(int);
     }
+    // size of forced bins
+    for (int i = 0; i < num_total_features_; ++i) {
+      size_of_header += forced_bin_bounds_[i].size() * sizeof(double) + sizeof(int);
+    }
     writer->Write(&size_of_header, sizeof(size_of_header));
     // write header
     writer->Write(&num_data_, sizeof(num_data_));
     writer->Write(&num_features_, sizeof(num_features_));
     writer->Write(&num_total_features_, sizeof(num_total_features_));
     writer->Write(&label_idx_, sizeof(label_idx_));
+    writer->Write(&max_bin_, sizeof(max_bin_));
+    writer->Write(&bin_construct_sample_cnt_, sizeof(bin_construct_sample_cnt_));
+    writer->Write(&min_data_in_bin_, sizeof(min_data_in_bin_));
+    writer->Write(&use_missing_, sizeof(use_missing_));
+    writer->Write(&zero_as_missing_, sizeof(zero_as_missing_));
+    writer->Write(&sparse_threshold_, sizeof(sparse_threshold_));
     writer->Write(used_feature_map_.data(), sizeof(int) * num_total_features_);
     writer->Write(&num_groups_, sizeof(num_groups_));
     writer->Write(real_feature_idx_.data(), sizeof(int) * num_features_);
@@ -603,6 +703,13 @@ void Dataset::SaveBinaryFile(const char* bin_filename) {
     if (ArrayArgs<double>::CheckAll(feature_penalty_, 1.0)) {
       feature_penalty_.clear();
     }
+    if (max_bin_by_feature_.empty()) {
+      ArrayArgs<int32_t>::Assign(&max_bin_by_feature_, -1, num_total_features_);
+    }
+    writer->Write(max_bin_by_feature_.data(), sizeof(int32_t) * num_total_features_);
+    if (ArrayArgs<int32_t>::CheckAll(max_bin_by_feature_, -1)) {
+      max_bin_by_feature_.clear();
+    }
     // write feature names
     for (int i = 0; i < num_total_features_; ++i) {
       int str_len = static_cast<int>(feature_names_[i].size());
@@ -610,6 +717,15 @@ void Dataset::SaveBinaryFile(const char* bin_filename) {
       const char* c_str = feature_names_[i].c_str();
       writer->Write(c_str, sizeof(char) * str_len);
     }
+    // write forced bins
+    for (int i = 0; i < num_total_features_; ++i) {
+      int num_bounds = static_cast<int>(forced_bin_bounds_[i].size());
+      writer->Write(&num_bounds, sizeof(int));
+      
+      for (size_t j = 0; j < forced_bin_bounds_[i].size(); ++j) {
+        writer->Write(&forced_bin_bounds_[i][j], sizeof(double));
+      }
+    }
 
     // get size of meta data
     size_t size_of_metadata = metadata_.SizesInByte();
@@ -628,15 +744,73 @@ void Dataset::SaveBinaryFile(const char* bin_filename) {
   }
 }
 
+void Dataset::DumpTextFile(const char* text_filename) {
+  FILE* file = NULL;
+#if _MSC_VER
+  fopen_s(&file, text_filename, "wt");
+#else
+  file = fopen(text_filename, "wt");
+#endif
+  fprintf(file, "num_features: %d\n", num_features_);
+  fprintf(file, "num_total_features: %d\n", num_total_features_);
+  fprintf(file, "num_groups: %d\n", num_groups_);
+  fprintf(file, "num_data: %d\n", num_data_);
+  fprintf(file, "feature_names: ");
+  for (auto n : feature_names_) {
+    fprintf(file, "%s, ", n.c_str());
+  }
+  fprintf(file, "\nmonotone_constraints: ");
+  for (auto i : monotone_types_) {
+    fprintf(file, "%d, ", i);
+  }
+  fprintf(file, "\nfeature_penalty: ");
+  for (auto i : feature_penalty_) {
+    fprintf(file, "%lf, ", i);
+  }
+  fprintf(file, "\nmax_bin_by_feature: ");
+  for (auto i : max_bin_by_feature_) {
+    fprintf(file, "%d, ", i);
+  }
+  fprintf(file, "\n");
+  for (auto n : feature_names_) {
+    fprintf(file, "%s, ", n.c_str());
+  }
+  fprintf(file, "\nforced_bins: ");
+  for (int i = 0; i < num_total_features_; ++i) {
+    fprintf(file, "\nfeature %d: ", i);
+    for (size_t j = 0; j < forced_bin_bounds_[i].size(); ++j) {
+      fprintf(file, "%lf, ", forced_bin_bounds_[i][j]);
+    }
+  }
+  std::vector<std::unique_ptr<BinIterator>> iterators;
+  iterators.reserve(num_features_);
+  for (int j = 0; j < num_features_; ++j) {
+    auto group_idx = feature2group_[j];
+    auto sub_idx = feature2subfeature_[j];
+    iterators.emplace_back(feature_groups_[group_idx]->SubFeatureIterator(sub_idx));
+  }
+  for (data_size_t i = 0; i < num_data_; ++i) {
+    fprintf(file, "\n");
+    for (int j = 0; j < num_total_features_; ++j) {
+      auto inner_feature_idx = used_feature_map_[j];
+      if (inner_feature_idx < 0) {
+        fprintf(file, "NA, ");
+      } else {
+        fprintf(file, "%d, ", iterators[inner_feature_idx]->RawGet(i));
+      }
+    }
+  }
+  fclose(file);
+}
+
 void Dataset::ConstructHistograms(const std::vector<int8_t>& is_feature_used,
                                   const data_size_t* data_indices, data_size_t num_data,
                                   int leaf_idx,
-                                  std::vector<std::unique_ptr<OrderedBin>>& ordered_bins,
+                                  std::vector<std::unique_ptr<OrderedBin>>* ordered_bins,
                                   const score_t* gradients, const score_t* hessians,
                                   score_t* ordered_gradients, score_t* ordered_hessians,
                                   bool is_constant_hessian,
                                   HistogramBinEntry* hist_data) const {
-
   if (leaf_idx < 0 || num_data < 0 || hist_data == nullptr) {
     return;
   }
@@ -684,9 +858,9 @@ void Dataset::ConstructHistograms(const std::vector<int8_t>& is_feature_used,
         // feature is not used
         auto data_ptr = hist_data + group_bin_boundaries_[group];
         const int num_bin = feature_groups_[group]->num_total_bin_;
-        std::memset((void*)(data_ptr + 1), 0, (num_bin - 1) * sizeof(HistogramBinEntry));
+        std::memset(reinterpret_cast<void*>(data_ptr + 1), 0, (num_bin - 1) * sizeof(HistogramBinEntry));
         // construct histograms for smaller leaf
-        if (ordered_bins[group] == nullptr) {
+        if (ordered_bins->at(group) == nullptr) {
           // if not use ordered bin
           feature_groups_[group]->bin_data_->ConstructHistogram(
             data_indices,
@@ -696,7 +870,7 @@ void Dataset::ConstructHistograms(const std::vector<int8_t>& is_feature_used,
             data_ptr);
         } else {
           // used ordered bin
-          ordered_bins[group]->ConstructHistogram(leaf_idx,
+          ordered_bins->at(group)->ConstructHistogram(leaf_idx,
                                                   gradients,
                                                   hessians,
                                                   data_ptr);
@@ -713,9 +887,9 @@ void Dataset::ConstructHistograms(const std::vector<int8_t>& is_feature_used,
         // feature is not used
         auto data_ptr = hist_data + group_bin_boundaries_[group];
         const int num_bin = feature_groups_[group]->num_total_bin_;
-        std::memset((void*)(data_ptr + 1), 0, (num_bin - 1) * sizeof(HistogramBinEntry));
+        std::memset(reinterpret_cast<void*>(data_ptr + 1), 0, (num_bin - 1) * sizeof(HistogramBinEntry));
         // construct histograms for smaller leaf
-        if (ordered_bins[group] == nullptr) {
+        if (ordered_bins->at(group) == nullptr) {
           // if not use ordered bin
           feature_groups_[group]->bin_data_->ConstructHistogram(
             data_indices,
@@ -724,7 +898,7 @@ void Dataset::ConstructHistograms(const std::vector<int8_t>& is_feature_used,
             data_ptr);
         } else {
           // used ordered bin
-          ordered_bins[group]->ConstructHistogram(leaf_idx,
+          ordered_bins->at(group)->ConstructHistogram(leaf_idx,
                                                   gradients,
                                                   data_ptr);
         }
@@ -746,9 +920,9 @@ void Dataset::ConstructHistograms(const std::vector<int8_t>& is_feature_used,
         // feature is not used
         auto data_ptr = hist_data + group_bin_boundaries_[group];
         const int num_bin = feature_groups_[group]->num_total_bin_;
-        std::memset((void*)(data_ptr + 1), 0, (num_bin - 1) * sizeof(HistogramBinEntry));
+        std::memset(reinterpret_cast<void*>(data_ptr + 1), 0, (num_bin - 1) * sizeof(HistogramBinEntry));
         // construct histograms for smaller leaf
-        if (ordered_bins[group] == nullptr) {
+        if (ordered_bins->at(group) == nullptr) {
           // if not use ordered bin
           feature_groups_[group]->bin_data_->ConstructHistogram(
             num_data,
@@ -757,7 +931,7 @@ void Dataset::ConstructHistograms(const std::vector<int8_t>& is_feature_used,
             data_ptr);
         } else {
           // used ordered bin
-          ordered_bins[group]->ConstructHistogram(leaf_idx,
+          ordered_bins->at(group)->ConstructHistogram(leaf_idx,
                                                   gradients,
                                                   hessians,
                                                   data_ptr);
@@ -774,9 +948,9 @@ void Dataset::ConstructHistograms(const std::vector<int8_t>& is_feature_used,
         // feature is not used
         auto data_ptr = hist_data + group_bin_boundaries_[group];
         const int num_bin = feature_groups_[group]->num_total_bin_;
-        std::memset((void*)(data_ptr + 1), 0, (num_bin - 1) * sizeof(HistogramBinEntry));
+        std::memset(reinterpret_cast<void*>(data_ptr + 1), 0, (num_bin - 1) * sizeof(HistogramBinEntry));
         // construct histograms for smaller leaf
-        if (ordered_bins[group] == nullptr) {
+        if (ordered_bins->at(group) == nullptr) {
           // if not use ordered bin
           feature_groups_[group]->bin_data_->ConstructHistogram(
             num_data,
@@ -784,7 +958,7 @@ void Dataset::ConstructHistograms(const std::vector<int8_t>& is_feature_used,
             data_ptr);
         } else {
           // used ordered bin
-          ordered_bins[group]->ConstructHistogram(leaf_idx,
+          ordered_bins->at(group)->ConstructHistogram(leaf_idx,
                                                   gradients,
                                                   data_ptr);
         }
@@ -820,4 +994,73 @@ void Dataset::FixHistogram(int feature_idx, double sum_gradient, double sum_hess
   }
 }
 
+template<typename T>
+void PushVector(std::vector<T>* dest, const std::vector<T>& src) {
+  dest->reserve(dest->size() + src.size());
+  for (auto i : src) {
+    dest->push_back(i);
+  }
+}
+
+template<typename T>
+void PushOffset(std::vector<T>* dest, const std::vector<T>& src, const T& offset) {
+  dest->reserve(dest->size() + src.size());
+  for (auto i : src) {
+    dest->push_back(i + offset);
+  }
+}
+
+template<typename T>
+void PushClearIfEmpty(std::vector<T>* dest, const size_t dest_len, const std::vector<T>& src, const size_t src_len, const T& deflt) {
+  if (!dest->empty() && !src.empty()) {
+    PushVector(dest, src);
+  } else if (!dest->empty() && src.empty()) {
+    for (size_t i = 0; i < src_len; ++i) {
+      dest->push_back(deflt);
+    }
+  } else if (dest->empty() && !src.empty()) {
+    for (size_t i = 0; i < dest_len; ++i) {
+      dest->push_back(deflt);
+    }
+    PushVector(dest, src);
+  }
+}
+
+void Dataset::addFeaturesFrom(Dataset* other) {
+  if (other->num_data_ != num_data_) {
+    throw std::runtime_error("Cannot add features from other Dataset with a different number of rows");
+  }
+  PushVector(&feature_names_, other->feature_names_);
+  PushVector(&feature2subfeature_, other->feature2subfeature_);
+  PushVector(&group_feature_cnt_, other->group_feature_cnt_);
+  PushVector(&forced_bin_bounds_, other->forced_bin_bounds_);
+  feature_groups_.reserve(other->feature_groups_.size());
+  for (auto& fg : other->feature_groups_) {
+    feature_groups_.emplace_back(new FeatureGroup(*fg));
+  }
+  for (auto feature_idx : other->used_feature_map_) {
+    if (feature_idx >= 0) {
+      used_feature_map_.push_back(feature_idx + num_features_);
+    } else {
+      used_feature_map_.push_back(-1);  // Unused feature.
+    }
+  }
+  PushOffset(&real_feature_idx_, other->real_feature_idx_, num_total_features_);
+  PushOffset(&feature2group_, other->feature2group_, num_groups_);
+  auto bin_offset = group_bin_boundaries_.back();
+  // Skip the leading 0 when copying group_bin_boundaries.
+  for (auto i = other->group_bin_boundaries_.begin()+1; i < other->group_bin_boundaries_.end(); ++i) {
+    group_bin_boundaries_.push_back(*i + bin_offset);
+  }
+  PushOffset(&group_feature_start_, other->group_feature_start_, num_features_);
+
+  PushClearIfEmpty(&monotone_types_, num_total_features_, other->monotone_types_, other->num_total_features_, (int8_t)0);
+  PushClearIfEmpty(&feature_penalty_, num_total_features_, other->feature_penalty_, other->num_total_features_, 1.0);
+  PushClearIfEmpty(&max_bin_by_feature_, num_total_features_, other->max_bin_by_feature_, other->num_total_features_, -1);
+
+  num_features_ += other->num_features_;
+  num_total_features_ += other->num_total_features_;
+  num_groups_ += other->num_groups_;
+}
+
 }  // namespace LightGBM
