@@ -1,18 +1,21 @@
+/*!
+ * Copyright (c) 2016 Microsoft Corporation. All rights reserved.
+ * Licensed under the MIT License. See LICENSE file in the project root for license information.
+ */
 #include <LightGBM/objective_function.h>
-#include "regression_objective.hpp"
+
 #include "binary_objective.hpp"
-#include "rank_objective.hpp"
 #include "multiclass_objective.hpp"
+#include "rank_objective.hpp"
+#include "regression_objective.hpp"
 #include "xentropy_objective.hpp"
 
 namespace LightGBM {
 
 ObjectiveFunction* ObjectiveFunction::CreateObjectiveFunction(const std::string& type, const Config& config) {
-  if (type == std::string("regression") || type == std::string("regression_l2")
-      || type == std::string("mean_squared_error") || type == std::string("mse") 
-      || type == std::string("l2_root") || type == std::string("root_mean_squared_error") || type == std::string("rmse")) {
+  if (type == std::string("regression")) {
     return new RegressionL2loss(config);
-  } else if (type == std::string("regression_l1") || type == std::string("mean_absolute_error")  || type == std::string("mae")) {
+  } else if (type == std::string("regression_l1")) {
     return new RegressionL1loss(config);
   } else if (type == std::string("quantile")) {
     return new RegressionQuantileloss(config);
@@ -26,21 +29,21 @@ ObjectiveFunction* ObjectiveFunction::CreateObjectiveFunction(const std::string&
     return new BinaryLogloss(config);
   } else if (type == std::string("lambdarank")) {
     return new LambdarankNDCG(config);
-  } else if (type == std::string("multiclass") || type == std::string("softmax")) {
+  } else if (type == std::string("multiclass")) {
     return new MulticlassSoftmax(config);
-  } else if (type == std::string("multiclassova") || type == std::string("multiclass_ova") || type == std::string("ova") || type == std::string("ovr")) {
+  } else if (type == std::string("multiclassova")) {
     return new MulticlassOVA(config);
-  } else if (type == std::string("xentropy") || type == std::string("cross_entropy")) {
+  } else if (type == std::string("cross_entropy")) {
     return new CrossEntropy(config);
-  } else if (type == std::string("xentlambda") || type == std::string("cross_entropy_lambda")) {
+  } else if (type == std::string("cross_entropy_lambda")) {
     return new CrossEntropyLambda(config);
-  } else if (type == std::string("mean_absolute_percentage_error") || type == std::string("mape")) {
+  } else if (type == std::string("mape")) {
     return new RegressionMAPELOSS(config);
   } else if (type == std::string("gamma")) {
     return new RegressionGammaLoss(config);
   } else if (type == std::string("tweedie")) {
     return new RegressionTweedieLoss(config);
-  } else if (type == std::string("none") || type == std::string("null") || type == std::string("custom") || type == std::string("na")) {
+  } else if (type == std::string("custom")) {
     return nullptr;
   }
   Log::Fatal("Unknown objective type name: %s", type.c_str());
@@ -69,17 +72,17 @@ ObjectiveFunction* ObjectiveFunction::CreateObjectiveFunction(const std::string&
     return new MulticlassSoftmax(strs);
   } else if (type == std::string("multiclassova")) {
     return new MulticlassOVA(strs);
-  } else if (type == std::string("xentropy") || type == std::string("cross_entropy")) {
+  } else if (type == std::string("cross_entropy")) {
     return new CrossEntropy(strs);
-  } else if (type == std::string("xentlambda") || type == std::string("cross_entropy_lambda")) {
+  } else if (type == std::string("cross_entropy_lambda")) {
     return new CrossEntropyLambda(strs);
-  } else if (type == std::string("mean_absolute_percentage_error") || type == std::string("mape")) {
+  } else if (type == std::string("mape")) {
     return new RegressionMAPELOSS(strs);
   } else if (type == std::string("gamma")) {
     return new RegressionGammaLoss(strs);
   } else if (type == std::string("tweedie")) {
     return new RegressionTweedieLoss(strs);
-  } else if (type == std::string("none") || type == std::string("null") || type == std::string("custom") || type == std::string("na")) {
+  } else if (type == std::string("custom")) {
     return nullptr;
   }
   Log::Fatal("Unknown objective type name: %s", type.c_str());
