@@ -1,14 +1,22 @@
+/*!
+ * Copyright (c) 2016 Microsoft Corporation. All rights reserved.
+ * Licensed under the MIT License. See LICENSE file in the project root for license information.
+ */
 #ifndef LIGHTGBM_OBJECTIVE_REGRESSION_OBJECTIVE_HPP_
 #define LIGHTGBM_OBJECTIVE_REGRESSION_OBJECTIVE_HPP_
 
-#include <LightGBM/objective_function.h>
 #include <LightGBM/meta.h>
-
+#include <LightGBM/objective_function.h>
 #include <LightGBM/utils/array_args.h>
 
+#include <string>
+#include <algorithm>
+#include <vector>
+
 namespace LightGBM {
 
 #define PercentileFun(T, data_reader, cnt_data, alpha) {\
+  if (cnt_data <= 1) { return  data_reader(0); }\
   std::vector<T> ref_data(cnt_data);\
   for (data_size_t i = 0; i < cnt_data; ++i) {\
     ref_data[i] = data_reader(i);\
@@ -36,6 +44,7 @@ namespace LightGBM {
 }\
 
 #define WeightedPercentileFun(T, data_reader, weight_reader, cnt_data, alpha) {\
+  if (cnt_data <= 1) { return  data_reader(0); }\
   std::vector<data_size_t> sorted_idx(cnt_data);\
   for (data_size_t i = 0; i < cnt_data; ++i) {\
     sorted_idx[i] = i;\
@@ -48,21 +57,26 @@ namespace LightGBM {
   }\
   double threshold = weighted_cdf[cnt_data - 1] * alpha;\
   size_t pos = std::upper_bound(weighted_cdf.begin(), weighted_cdf.end(), threshold) - weighted_cdf.begin();\
-  if (pos == 0 || pos == cnt_data - 1) {\
+  pos = std::min(pos, static_cast<size_t>(cnt_data -1));\
+  if (pos == 0 || pos ==  static_cast<size_t>(cnt_data - 1)) {\
     return data_reader(sorted_idx[pos]);\
   }\
   CHECK(threshold >= weighted_cdf[pos - 1]);\
   CHECK(threshold < weighted_cdf[pos]);\
   T v1 = data_reader(sorted_idx[pos - 1]);\
   T v2 = data_reader(sorted_idx[pos]);\
-  return static_cast<T>((threshold - weighted_cdf[pos]) / (weighted_cdf[pos + 1] - weighted_cdf[pos]) * (v2 - v1) + v1);\
+  if (weighted_cdf[pos + 1] - weighted_cdf[pos] >= 1.0f) {\
+    return static_cast<T>((threshold - weighted_cdf[pos]) / (weighted_cdf[pos + 1] - weighted_cdf[pos]) * (v2 - v1) + v1); \
+  } else {\
+    return static_cast<T>(v2);\
+  }\
 }\
 
 /*!
 * \brief Objective function for regression
 */
 class RegressionL2loss: public ObjectiveFunction {
-public:
+ public:
   explicit RegressionL2loss(const Config& config) {
     sqrt_ = config.reg_sqrt;
   }
@@ -75,7 +89,7 @@ public:
       }
     }
   }
-  
+
   ~RegressionL2loss() {
   }
 
@@ -143,7 +157,7 @@ public:
     double suml = 0.0f;
     double sumw = 0.0f;
     if (weights_ != nullptr) {
-      #pragma omp parallel for schedule(static) reduction(+:suml,sumw)
+      #pragma omp parallel for schedule(static) reduction(+:suml, sumw)
       for (data_size_t i = 0; i < num_data_; ++i) {
         suml += label_[i] * weights_[i];
         sumw += weights_[i];
@@ -158,7 +172,7 @@ public:
     return suml / sumw;
   }
 
-protected:
+ protected:
   bool sqrt_;
   /*! \brief Number of data */
   data_size_t num_data_;
@@ -173,7 +187,7 @@ protected:
 * \brief L1 regression loss
 */
 class RegressionL1loss: public RegressionL2loss {
-public:
+ public:
   explicit RegressionL1loss(const Config& config): RegressionL2loss(config) {
   }
 
@@ -218,30 +232,30 @@ public:
 
   bool IsRenewTreeOutput() const override { return true; }
 
-  double RenewTreeOutput(double, const double* pred, 
+  double RenewTreeOutput(double, std::function<double(const label_t*, int)> residual_getter,
                          const data_size_t* index_mapper,
                          const data_size_t* bagging_mapper,
                          data_size_t num_data_in_leaf) const override {
     const double alpha = 0.5;
     if (weights_ == nullptr) {
       if (bagging_mapper == nullptr) {
-        #define data_reader(i) (label_[index_mapper[i]] - pred[index_mapper[i]])
+        #define data_reader(i) (residual_getter(label_, index_mapper[i]))
         PercentileFun(double, data_reader, num_data_in_leaf, alpha);
         #undef data_reader
       } else {
-        #define data_reader(i) (label_[bagging_mapper[index_mapper[i]]] - pred[bagging_mapper[index_mapper[i]]])
+        #define data_reader(i) (residual_getter(label_, bagging_mapper[index_mapper[i]]))
         PercentileFun(double, data_reader, num_data_in_leaf, alpha);
         #undef data_reader
       }
     } else {
       if (bagging_mapper == nullptr) {
-        #define data_reader(i) (label_[index_mapper[i]] - pred[index_mapper[i]])
+        #define data_reader(i) (residual_getter(label_, index_mapper[i]))
         #define weight_reader(i) (weights_[index_mapper[i]])
         WeightedPercentileFun(double, data_reader, weight_reader, num_data_in_leaf, alpha);
         #undef data_reader
         #undef weight_reader
       } else {
-        #define data_reader(i) (label_[bagging_mapper[index_mapper[i]]] - pred[bagging_mapper[index_mapper[i]]])
+        #define data_reader(i) (residual_getter(label_, bagging_mapper[index_mapper[i]]))
         #define weight_reader(i) (weights_[bagging_mapper[index_mapper[i]]])
         WeightedPercentileFun(double, data_reader, weight_reader, num_data_in_leaf, alpha);
         #undef data_reader
@@ -259,7 +273,7 @@ public:
 * \brief Huber regression loss
 */
 class RegressionHuberLoss: public RegressionL2loss {
-public:
+ public:
   explicit RegressionHuberLoss(const Config& config): RegressionL2loss(config) {
     alpha_ = static_cast<double>(config.alpha);
     if (sqrt_) {
@@ -313,7 +327,7 @@ public:
     return false;
   }
 
-private:
+ private:
   /*! \brief delta for Huber loss */
   double alpha_;
 };
@@ -321,13 +335,12 @@ private:
 
 // http://research.microsoft.com/en-us/um/people/zhang/INRIA/Publis/Tutorial-Estim/node24.html
 class RegressionFairLoss: public RegressionL2loss {
-public:
+ public:
   explicit RegressionFairLoss(const Config& config): RegressionL2loss(config) {
     c_ = static_cast<double>(config.fair_c);
   }
 
   explicit RegressionFairLoss(const std::vector<std::string>& strs): RegressionL2loss(strs) {
-
   }
 
   ~RegressionFairLoss() {}
@@ -359,7 +372,7 @@ public:
     return false;
   }
 
-private:
+ private:
   /*! \brief c for Fair loss */
   double c_;
 };
@@ -369,7 +382,7 @@ private:
 * \brief Objective function for Poisson regression
 */
 class RegressionPoissonLoss: public RegressionL2loss {
-public:
+ public:
   explicit RegressionPoissonLoss(const Config& config): RegressionL2loss(config) {
     max_delta_step_ = static_cast<double>(config.poisson_max_delta_step);
     if (sqrt_) {
@@ -379,7 +392,6 @@ public:
   }
 
   explicit RegressionPoissonLoss(const std::vector<std::string>& strs): RegressionL2loss(strs) {
-
   }
 
   ~RegressionPoissonLoss() {}
@@ -393,7 +405,7 @@ public:
     // Safety check of labels
     label_t miny;
     double sumy;
-    Common::ObtainMinMaxSum(label_, num_data_, &miny, (label_t*)nullptr, &sumy);
+    Common::ObtainMinMaxSum(label_, num_data_, &miny, static_cast<label_t*>(nullptr), &sumy);
     if (miny < 0.0f) {
       Log::Fatal("[%s]: at least one target label is negative", GetName());
     }
@@ -437,27 +449,26 @@ public:
   }
 
   double BoostFromScore(int) const override {
-    return std::log(RegressionL2loss::BoostFromScore(0));
+    return Common::SafeLog(RegressionL2loss::BoostFromScore(0));
   }
 
   bool IsConstantHessian() const override {
     return false;
   }
 
-private:
+ private:
   /*! \brief used to safeguard optimization */
   double max_delta_step_;
 };
 
 class RegressionQuantileloss : public RegressionL2loss {
-public:
+ public:
   explicit RegressionQuantileloss(const Config& config): RegressionL2loss(config) {
     alpha_ = static_cast<score_t>(config.alpha);
     CHECK(alpha_ > 0 && alpha_ < 1);
   }
 
   explicit RegressionQuantileloss(const std::vector<std::string>& strs): RegressionL2loss(strs) {
-
   }
 
   ~RegressionQuantileloss() {}
@@ -509,29 +520,29 @@ public:
 
   bool IsRenewTreeOutput() const override { return true; }
 
-  double RenewTreeOutput(double, const double* pred,
+  double RenewTreeOutput(double, std::function<double(const label_t*, int)> residual_getter,
                          const data_size_t* index_mapper,
                          const data_size_t* bagging_mapper,
                          data_size_t num_data_in_leaf) const override {
     if (weights_ == nullptr) {
       if (bagging_mapper == nullptr) {
-        #define data_reader(i) (label_[index_mapper[i]] - pred[index_mapper[i]])
+        #define data_reader(i) (residual_getter(label_, index_mapper[i]))
         PercentileFun(double, data_reader, num_data_in_leaf, alpha_);
         #undef data_reader
       } else {
-        #define data_reader(i) (label_[bagging_mapper[index_mapper[i]]] - pred[bagging_mapper[index_mapper[i]]])
+        #define data_reader(i) (residual_getter(label_, bagging_mapper[index_mapper[i]]))
         PercentileFun(double, data_reader, num_data_in_leaf, alpha_);
         #undef data_reader
       }
     } else {
       if (bagging_mapper == nullptr) {
-        #define data_reader(i) (label_[index_mapper[i]] - pred[index_mapper[i]])
+        #define data_reader(i) (residual_getter(label_, index_mapper[i]))
         #define weight_reader(i) (weights_[index_mapper[i]])
         WeightedPercentileFun(double, data_reader, weight_reader, num_data_in_leaf, alpha_);
         #undef data_reader
         #undef weight_reader
       } else {
-        #define data_reader(i) (label_[bagging_mapper[index_mapper[i]]] - pred[bagging_mapper[index_mapper[i]]])
+        #define data_reader(i) (residual_getter(label_, bagging_mapper[index_mapper[i]]))
         #define weight_reader(i) (weights_[bagging_mapper[index_mapper[i]]])
         WeightedPercentileFun(double, data_reader, weight_reader, num_data_in_leaf, alpha_);
         #undef data_reader
@@ -540,7 +551,7 @@ public:
     }
   }
 
-private:
+ private:
   score_t alpha_;
 };
 
@@ -549,12 +560,11 @@ private:
 * \brief Mape Regression Loss
 */
 class RegressionMAPELOSS : public RegressionL1loss {
-public:
+ public:
   explicit RegressionMAPELOSS(const Config& config) : RegressionL1loss(config) {
   }
 
   explicit RegressionMAPELOSS(const std::vector<std::string>& strs) : RegressionL1loss(strs) {
-
   }
 
   ~RegressionMAPELOSS() {}
@@ -611,19 +621,19 @@ public:
 
   bool IsRenewTreeOutput() const override { return true; }
 
-  double RenewTreeOutput(double, const double* pred,
+  double RenewTreeOutput(double, std::function<double(const label_t*, int)> residual_getter,
                          const data_size_t* index_mapper,
                          const data_size_t* bagging_mapper,
                          data_size_t num_data_in_leaf) const override {
     const double alpha = 0.5;
     if (bagging_mapper == nullptr) {
-      #define data_reader(i) (label_[index_mapper[i]] - pred[index_mapper[i]])
+      #define data_reader(i) (residual_getter(label_, index_mapper[i]))
       #define weight_reader(i) (label_weight_[index_mapper[i]])
       WeightedPercentileFun(double, data_reader, weight_reader, num_data_in_leaf, alpha);
       #undef data_reader
       #undef weight_reader
     } else {
-      #define data_reader(i) (label_[bagging_mapper[index_mapper[i]]] - pred[bagging_mapper[index_mapper[i]]])
+      #define data_reader(i) (residual_getter(label_, bagging_mapper[index_mapper[i]]))
       #define weight_reader(i) (label_weight_[bagging_mapper[index_mapper[i]]])
       WeightedPercentileFun(double, data_reader, weight_reader, num_data_in_leaf, alpha);
       #undef data_reader
@@ -639,9 +649,8 @@ public:
     return true;
   }
 
-private:
+ private:
   std::vector<label_t> label_weight_;
-
 };
 
 
@@ -650,12 +659,11 @@ private:
 * \brief Objective function for Gamma regression
 */
 class RegressionGammaLoss : public RegressionPoissonLoss {
-public:
+ public:
   explicit RegressionGammaLoss(const Config& config) : RegressionPoissonLoss(config) {
   }
 
   explicit RegressionGammaLoss(const std::vector<std::string>& strs) : RegressionPoissonLoss(strs) {
-
   }
 
   ~RegressionGammaLoss() {}
@@ -680,20 +688,18 @@ public:
   const char* GetName() const override {
     return "gamma";
   }
- 
 };
 
 /*!
 * \brief Objective function for Tweedie regression
 */
 class RegressionTweedieLoss: public RegressionPoissonLoss {
-public:
+ public:
   explicit RegressionTweedieLoss(const Config& config) : RegressionPoissonLoss(config) {
     rho_ = config.tweedie_variance_power;
   }
 
   explicit RegressionTweedieLoss(const std::vector<std::string>& strs) : RegressionPoissonLoss(strs) {
-
   }
 
   ~RegressionTweedieLoss() {}
@@ -704,7 +710,7 @@ public:
       #pragma omp parallel for schedule(static)
       for (data_size_t i = 0; i < num_data_; ++i) {
         gradients[i] = static_cast<score_t>(-label_[i] * std::exp((1 - rho_) * score[i]) + std::exp((2 - rho_) * score[i]));
-        hessians[i] = static_cast<score_t>(-label_[i] * (1 - rho_) * std::exp((1 - rho_) * score[i]) + 
+        hessians[i] = static_cast<score_t>(-label_[i] * (1 - rho_) * std::exp((1 - rho_) * score[i]) +
           (2 - rho_) * std::exp((2 - rho_) * score[i]));
       }
     } else {
@@ -720,7 +726,8 @@ public:
   const char* GetName() const override {
     return "tweedie";
   }
-private:
+
+ private:
   double rho_;
 };
 
